package EngineThrottleControl "Package containing the engine throttle control example"
  model EngineThrottleControlExample "Model of the engine throttle control"
    extends Modelica.Icons.Example;
    Modelica.Blocks.Sources.Step speedRef(startTime = 5, offset = 207.34, height = 103.67) annotation(Placement(transformation(extent = {{-90, 6}, {-70, 26}})));
    Components.SpeedControl speedControl annotation(Placement(transformation(extent = {{-32, -5}, {0, 25}})));
    Modelica.Clocked.RealSignals.Sampler.Sample sample1 annotation(Placement(transformation(extent = {{-60, 9}, {-46, 23}})));
    Modelica.Clocked.RealSignals.Sampler.Hold hold1(y_start = 8.9) annotation(Placement(transformation(extent = {{8, 4}, {20, 16}})));
    Modelica.Clocked.ClockSignals.Clocks.Rotational.FixedRotationalClock rotationalClock(trigger_interval(displayUnit = "deg") = 3.1415926535898) annotation(Placement(transformation(extent = {{20, -50}, {0, -30}})));
    Components.Engine engine(crankshaftPositionEvent(trigger_interval = rotationalClock.trigger_interval)) annotation(Placement(transformation(extent = {{30, -4}, {58, 24}})));
    Modelica.Blocks.Sources.Step step(height = -5, offset = 25, startTime = 2) annotation(Placement(transformation(extent = {{148, 17}, {134, 31}})));
    Modelica.Blocks.Sources.Step step1(height = 5, offset = 0, startTime = 8) annotation(Placement(transformation(extent = {{148, -10}, {134, 4}})));
    Modelica.Blocks.Math.Add add(k1 = -1, k2 = -1) annotation(Placement(transformation(extent = {{122, 4}, {110, 16}})));
    Modelica.Mechanics.Rotational.Sources.Torque torque2(useSupport = false) annotation(Placement(transformation(extent = {{90, 0}, {70, 20}})));
    Modelica.Mechanics.Rotational.Sensors.AngleSensor angleSensor annotation(Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {64, -22})));
    Modelica.Blocks.Continuous.Der derivative annotation(Placement(transformation(extent = {{20, -90}, {0, -70}})));
    Modelica.Clocked.RealSignals.Sampler.SampleClocked sample2 annotation(Placement(transformation(extent = {{-24, -74}, {-36, -86}})));
  equation
    connect(speedRef.y, sample1.u) annotation(Line(points = {{-69, 16}, {-61.4, 16}}, color = {0, 0, 127}));
    connect(sample1.y, speedControl.N_des) annotation(Line(points = {{-45.3, 16}, {-35.2, 16}}, color = {0, 0, 127}));
    connect(speedControl.Theta, hold1.u) annotation(Line(points = {{1.6, 10}, {6.8, 10}}, color = {0, 0, 127}));
    connect(hold1.y, engine.Theta) annotation(Line(points = {{20.6, 10}, {27.2, 10}}, color = {0, 0, 127}));
    connect(torque2.flange, engine.flange_b) annotation(Line(points = {{70, 10}, {58.56, 10}}));
    connect(add.y, torque2.tau) annotation(Line(points = {{109.4, 10}, {92, 10}}, color = {0, 0, 127}));
    connect(step1.y, add.u2) annotation(Line(points = {{133.3, -3}, {128, -3}, {128, 6.4}, {123.2, 6.4}}, color = {0, 0, 127}));
    connect(step.y, add.u1) annotation(Line(points = {{133.3, 24}, {128, 24}, {128, 13.6}, {123.2, 13.6}}, color = {0, 0, 127}));
    connect(engine.flange_b, angleSensor.flange) annotation(Line(points = {{58.56, 10}, {64, 10}, {64, -12}}));
    connect(angleSensor.phi, derivative.u) annotation(Line(points = {{64, -33}, {64, -80}, {22, -80}}, color = {0, 0, 127}));
    connect(derivative.y, sample2.u) annotation(Line(points = {{-1, -80}, {-22.8, -80}}, color = {0, 0, 127}));
    connect(sample2.y, speedControl.N) annotation(Line(points = {{-36.6, -80}, {-50, -80}, {-50, 1}, {-35.2, 1}}, color = {0, 0, 127}));
    connect(rotationalClock.y, sample2.clock) annotation(Line(points = {{-1, -40}, {-30, -40}, {-30, -72.8}}, color = {175, 175, 175}, pattern = LinePattern.Dot, thickness = 0.5));
    connect(angleSensor.phi, rotationalClock.angle) annotation(Line(points = {{64, -33}, {64, -40}, {22, -40}}, color = {0, 0, 127}));
    annotation(experiment(StopTime = 15), __Wolfram(PlotSet(plots = {Plot(name = "SpeedResponse", preferred = true, subPlots = {SubPlot(curves = {Curve(x = time, y = speedControl.N_des), Curve(x = time, y = speedControl.N)}), SubPlot(curves = {Curve(x = time, y = engine.torque.tau), Curve(x = time, y = torque2.flange.tau)})})})), Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {160, 50}}, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {0, 0, 255}, extent = {{104, -22}, {154, 36}}), Text(visible = true, textColor = {0, 0, 255}, extent = {{110, -20}, {152, -16}}, textString = "Load torque")}), Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}})), Documentation(info = "<html>


    <p>
This example shows how to control the engine speed of an internal combustion engine using a throttle actuator. The controller execution is synchronized to the engine crankshaft angle which leads to reduction in influence of disturbances such as changes in load torque.
</p>
<p>
Check the model diagram:
</p><p>
<img src=\"modelica://EngineThrottleControl/DocumentationFiles/DiagramModel.png\" alt=\"DiagramModel\"/>
</p><p>
The speedControl is the discrete control system whose boundaries are defined by sampl1, sample2 and hold1. The sampling is done by the rotationalClock which is an event-based clock that is triggered every 180° rotation of the crankshaft angle.
</p>
    <h4>Simulation Results</h4>
    <p>
The engine speed stabilizes quickly to its stable speed reducing the influence of load torque fluctuation.
</p><p>
<img src=\"modelica://EngineThrottleControl/DocumentationFiles/SimulationResults.png\" alt=\"SimulationResults\"/>
</p>

<p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at&nbsp;<a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>

<p>For the full example, open the accompanying notebook <a href=\"modelica://EngineThrottleControl/EngineThrottleControl.nb\">EngineThrottleControl.nb</a>.</p></html>
</html>"));
  end EngineThrottleControlExample;

  package Components "Package containing components used by the example"
    extends Modelica.Icons.Package;

    block ThrottleBody "Basic throttle body equations"
      extends Modelica.Blocks.Icons.Block;
      parameter Real P_0 = 1 "Atmospheric pressure, bar";
    protected
      Real m_ai(start = 0, fixed = true) "mass, g";
      Real f_Theta "auxiliary variable";
      Real g_Pm "auxiliary variable";
    public
      Modelica.Blocks.Interfaces.RealInput Theta "throttle angle, degrees" annotation(Placement(transformation(extent = {{-140, 40}, {-100, 80}})));
      Modelica.Blocks.Interfaces.RealInput Pm "Intake manifold presure, bar" annotation(Placement(transformation(extent = {{-140, -80}, {-100, -40}})));
      Modelica.Blocks.Interfaces.RealOutput m_ai_dot "Mass flow rate of air into manifold (g/s)" annotation(Placement(transformation(extent = {{100, -10}, {120, 10}})));
    protected
      Real tmp1;
      Real tmp2;
      Real pratio;
      Real gpratio;
    equation
      der(m_ai) = f_Theta * g_Pm;
      f_Theta = 2.821 - 0.05231 * Theta + 0.10299 * Theta ^ 2 - 0.00063 * Theta ^ 3;
      // different to paper 0.0063*Theta^3
      // Note: If the flow direction changes, i.e. Pm>P_0 the logic will fail since sqrt(-x)
      tmp1 = Pm / P_0;
      tmp2 = P_0 / Pm;
      pratio = min(tmp1, tmp2);
      gpratio = if pratio >= 0.5 then 2 * sqrt(pratio - pratio ^ 2) else 1;
      g_Pm = sign(P_0 - Pm) * gpratio;
      /*
g_Pm = if Pm <= P_0/2 then 1.0 else 2/P_0*sqrt(Pm*P_0 - Pm^2);
*/
      m_ai_dot = der(m_ai);
      annotation(Documentation(info = "<html>

    <p>
Basic throttle body equations
</p></html>"), Diagram(graphics));
    end ThrottleBody;

    block IntakeManifold "Dynamics of the intake manifold"
      extends Modelica.Blocks.Icons.Block;
      parameter Real RTVmRatio = 0.41328 "RT/V_m TODO: What are sensible individual values of R,T, and V_m! (took same value like SIMULINK example for comparison reasons)";
      parameter Real P_0 = 0.543 "Initial value for P_m, (bar) (took same value like SIMULINK example for comparison reasons)";
      Modelica.Blocks.Interfaces.RealInput m_ai_dot "Mass flow rate of air into manifold (g/s) " annotation(Placement(transformation(extent = {{-140, 40}, {-100, 80}})));
      Modelica.Blocks.Interfaces.RealInput N "Engine speed (rad/sec)" annotation(Placement(transformation(extent = {{-140, -80}, {-100, -40}})));
      Modelica.Blocks.Interfaces.RealOutput m_ao_dot "Mass flow rate of air out of manifold (g/s)" annotation(Placement(transformation(extent = {{100, 50}, {120, 70}})));
      Modelica.Blocks.Interfaces.RealOutput P_m(start = P_0, fixed = true) "Intake manifold presure, (bar)" annotation(Placement(transformation(extent = {{100, -70}, {120, -50}})));
    equation
      der(P_m) = RTVmRatio * (m_ai_dot - m_ao_dot);
      // TODO adapt so that der(m_ai) and der(m_ao) is used!
      m_ao_dot = -0.366 + 0.08979 * N * P_m - 0.0337 * N * P_m ^ 2 + 0.0001 * N ^ 2 * P_m;
      annotation(Documentation(info = "<html>

    <p>
Dynamics of the intake manifold
</p></html>"), Diagram(graphics));
    end IntakeManifold;

    block TorqueGeneration "Torque generation"
      extends Modelica.Blocks.Icons.Block;
      parameter Real AFR = 14.6 "air-fuel ratio (took same value like SIMULINK example for comparison reasons)";
      parameter Real sigma = 15.0 "spark advance, BTDC (took same value like SIMULINK example for comparison reasons)";
      Modelica.Blocks.Interfaces.RealInput m_a "Mass of air charge in cylinder (delayed of 180deg crankshaft rotation) (g) " annotation(Placement(transformation(extent = {{-140, 40}, {-100, 80}})));
      Modelica.Blocks.Interfaces.RealInput N "Engine speed (rad/sec)" annotation(Placement(transformation(extent = {{-140, -80}, {-100, -40}})));
      Modelica.Blocks.Interfaces.RealOutput T_torque_e "Torque generated by engine" annotation(Placement(transformation(extent = {{100, -10}, {120, 10}})));
    equation
      T_torque_e = -181.3 + 379.36 * m_a + 21.91 * AFR - 0.85 * AFR ^ 2 + 0.26 * sigma - 0.0028 * sigma ^ 2 + 0.027 * N - 0.000107 * N ^ 2 + 0.00048 * N * sigma + 2.55 * sigma * m_a - 0.05 * sigma ^ 2 * m_a;
      annotation(Documentation(info = "<html>

    <p>
Torque generation
</p></html>"), Diagram(graphics));
    end TorqueGeneration;

    block SpeedControl "Discrete control of crankshaft speed by throttle actuation"
      parameter Real k_p = 0.0614 "Proportional gain";
      parameter Real k_I = 0.0723 "Integral gain";
      Modelica.Blocks.Interfaces.RealInput N_des "Desired speed, (rad/s)" annotation(Placement(transformation(extent = {{-140, 20}, {-100, 60}})));
      Modelica.Blocks.Interfaces.RealInput N "Measured speed, (rad/s)" annotation(Placement(transformation(extent = {{-140, -80}, {-100, -40}})));
      Modelica.Blocks.Interfaces.RealOutput Theta(start = 8.9) "throttle angle, degrees" annotation(Placement(transformation(extent = {{100, -10}, {120, 10}})));
    protected
      Real x_d(start = 8.973, fixed = true);
      /*
initial equation
  N_des = pre(N_des);
  N = pre(N);
*/
    equation
      when Clock() then
        x_d = previous(x_d) + k_I * interval() * (previous(N_des) - previous(N));
        Theta = k_p * (N_des - N) + x_d;
      end when;
      annotation(Documentation(info = "<html>

    <p>
Discrete control of crankshaft speed by throttle actuation
</p></html>"), Diagram(graphics), Icon(graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-4, -36}, {74, 20}, {74, -2}, {-4, -58}, {-4, -36}}, lineColor = {95, 95, 95}, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Polygon(points = {{8, -34}, {28, -44}, {20, -50}, {0, -40}, {8, -34}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{20, -50}, {28, -44}, {32, -52}, {24, -58}, {20, -50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{24, -58}, {32, -52}, {32, -74}, {24, -80}, {24, -58}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{0, -40}, {20, -50}, {24, -58}, {24, -80}, {22, -80}, {22, -58}, {20, -54}, {0, -44}, {0, -40}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-14, 44}, {-18, 36}, {-10, 42}, {-8, 48}, {-14, 44}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Polygon(points = {{-18, 36}, {-10, 42}, {-10, 18}, {-18, 12}, {-18, 36}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-20, 14}, {-20, 36}, {-16, 44}, {4, 34}, {4, 30}, {-16, 40}, {-18, 36}, {-18, 12}, {-20, 14}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-8, 50}, {12, 40}, {4, 34}, {-16, 44}, {-8, 50}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Text(extent = {{-150, 90}, {150, 60}}, textString = "%name", lineColor = {0, 0, 255}), Polygon(points = {{28, -18}, {48, -28}, {40, -34}, {20, -24}, {28, -18}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{40, -34}, {48, -28}, {52, -36}, {44, -42}, {40, -34}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{44, -42}, {52, -36}, {52, -58}, {44, -64}, {44, -42}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{20, -24}, {40, -34}, {44, -42}, {44, -64}, {42, -64}, {42, -42}, {40, -38}, {20, -28}, {20, -24}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{48, -4}, {68, -14}, {60, -20}, {40, -10}, {48, -4}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{60, -20}, {68, -14}, {72, -22}, {64, -28}, {60, -20}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{64, -28}, {72, -22}, {72, -44}, {64, -50}, {64, -28}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{40, -10}, {60, -20}, {64, -28}, {64, -50}, {62, -50}, {62, -28}, {60, -24}, {40, -14}, {40, -10}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{68, 10}, {88, 0}, {80, -6}, {60, 4}, {68, 10}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{80, -6}, {88, 0}, {92, -8}, {84, -14}, {80, -6}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{84, -14}, {92, -8}, {92, -30}, {84, -36}, {84, -14}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{60, 4}, {80, -6}, {84, -14}, {84, -36}, {82, -36}, {82, -14}, {80, -10}, {60, 0}, {60, 4}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-34, 30}, {-38, 22}, {-30, 28}, {-28, 34}, {-34, 30}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Polygon(points = {{-38, 22}, {-30, 28}, {-30, 4}, {-38, -2}, {-38, 22}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-40, 0}, {-40, 22}, {-36, 30}, {-16, 20}, {-16, 16}, {-36, 26}, {-38, 22}, {-38, -2}, {-40, 0}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-28, 36}, {-8, 26}, {-16, 20}, {-36, 30}, {-28, 36}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-54, 16}, {-58, 8}, {-50, 14}, {-48, 20}, {-54, 16}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Polygon(points = {{-58, 8}, {-50, 14}, {-50, -10}, {-58, -16}, {-58, 8}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-60, -14}, {-60, 8}, {-56, 16}, {-36, 6}, {-36, 2}, {-56, 12}, {-58, 8}, {-58, -16}, {-60, -14}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-48, 22}, {-28, 12}, {-36, 6}, {-56, 16}, {-48, 22}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-74, 0}, {-78, -8}, {-70, -2}, {-68, 4}, {-74, 0}}, lineColor = {128, 128, 128}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Polygon(points = {{-78, -8}, {-70, -2}, {-70, -26}, {-78, -32}, {-78, -8}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 127}, fillPattern = FillPattern.Solid), Polygon(points = {{-80, -30}, {-80, -8}, {-76, 0}, {-56, -10}, {-56, -14}, {-76, -4}, {-78, -8}, {-78, -32}, {-80, -30}}, lineColor = {0, 0, 255}, fillColor = {191, 191, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-68, 6}, {-48, -4}, {-56, -10}, {-76, 0}, {-68, 6}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid), Polygon(points = {{-64, -4}, {-4, -36}, {-4, -58}, {-64, -26}, {-64, -4}}, lineColor = {95, 95, 95}, fillColor = {75, 75, 75}, fillPattern = FillPattern.Solid), Polygon(points = {{-64, -4}, {-4, -36}, {74, 20}, {14, 52}, {-64, -4}}, lineColor = {95, 95, 95}, fillColor = {160, 160, 164}, fillPattern = FillPattern.Solid)}));
    end SpeedControl;

    block CylinderAirCharge "Integrates the air mass flow into a cylinder. After the charge for one cylinder is complete, resets the mass to 0"
      extends Modelica.Blocks.Icons.Block;
      Modelica.Blocks.Interfaces.RealOutput m_a "Mass of an cylinder air charge (g)" annotation(Placement(transformation(extent = {{100, -10}, {120, 10}})));
      Modelica.Blocks.Interfaces.RealInput m_ao_dot "Mass flow rate of air out of manifold (g/s)" annotation(Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
      Modelica.Clocked.ClockSignals.Interfaces.ClockInput clock annotation(Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {0, -120})));
    protected
      Real m_a_total(start = 0, fixed = true) "Total amount of air that was sucked in";
      Real m_a_last "Amount of air sucked in until last clock event";
    equation
      der(m_a_total) = m_ao_dot;
      m_a = m_a_total - hold(m_a_last);
      when clock then
        m_a_last = sample(m_a_total);
      end when;
      annotation(Documentation(info = "<html>

    <p>
Integrates the air mass flow into a cylinder. After the charge for one cylinder is complete, resets the mass to 0
</p></html>"), Diagram(graphics));
    end CylinderAirCharge;

    block InductionToPowerDelay "Accounts for the induction-to-power stroke lag"
      extends Modelica.Blocks.Icons.Block;
      Modelica.Blocks.Interfaces.RealInput m_a "Mass of cylinder air charge (g)" annotation(Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
      Modelica.Blocks.Interfaces.RealOutput m_a_delayed "180deg delayed mass of cylinder air charge (g)" annotation(Placement(transformation(extent = {{100, -10}, {120, 10}})));
    protected
      Real m_a_sampled(start = 0.152, fixed = true);
      Real m_a_delayedSampled(start = 0.152, fixed = true);
    public
      Modelica.Clocked.ClockSignals.Interfaces.ClockInput clock annotation(Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {0, -120})));
    equation
      when clock then
        m_a_sampled = sample(m_a);
        m_a_delayedSampled = previous(m_a_sampled);
      end when;
      m_a_delayed = hold(m_a_delayedSampled);
      annotation(Documentation(info = "<html>

    <p>
Accounts for the induction-to-power stroke lag
</p></html>"), Diagram(graphics));
    end InductionToPowerDelay;

    model Engine "Internal combustion engine"
      Modelica.Blocks.Interfaces.RealInput Theta "throttle angle, degrees" annotation(Placement(transformation(extent = {{-140, -20}, {-100, 20}})));
      Modelica.Clocked.Examples.Systems.Utilities.ComponentsThrottleControl.ThrottleBody throttleBody annotation(Placement(transformation(extent = {{-78, 20}, {-58, 40}}, origin = {0, 10}, rotation = 0), visible = true));
      Modelica.Clocked.Examples.Systems.Utilities.ComponentsThrottleControl.IntakeManifold intakeManifold annotation(Placement(transformation(extent = {{-78, -20}, {-58, 0}})));
      Modelica.Clocked.Examples.Systems.Utilities.ComponentsThrottleControl.TorqueGeneration torqueGeneration annotation(Placement(transformation(extent = {{18, 28}, {38, 48}}, origin = {0.307, 2}, rotation = 0), visible = true));
      Modelica.Clocked.ClockSignals.Clocks.Rotational.FixedRotationalClock crankshaftPositionEvent annotation(Placement(transformation(extent = {{70, -74}, {50, -54}})));
      Modelica.Clocked.Examples.Systems.Utilities.ComponentsThrottleControl.InductionToPowerDelay inductionToPowerDelay annotation(Placement(transformation(extent = {{-22, -20}, {-2, 0}}, origin = {-0.268, -23.052}, rotation = 0), visible = true));
      Modelica.Clocked.Examples.Systems.Utilities.ComponentsThrottleControl.CylinderAirCharge cylinderAirCharge annotation(Placement(transformation(extent = {{-50, -20}, {-30, 0}}, origin = {0, 5.4}, rotation = 0), visible = true));
      Modelica.Mechanics.Rotational.Sources.Torque torque(useSupport = false) annotation(Placement(transformation(extent = {{40, -8}, {56, 8}})));
      Modelica.Mechanics.Rotational.Components.Inertia inertia1(J = 0.14, w(fixed = true, start = 209), phi(fixed = true, start = 0)) annotation(Placement(transformation(extent = {{64, -8}, {80, 8}})));
      Modelica.Mechanics.Rotational.Sensors.SpeedSensor speedSensor annotation(Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {88, -18})));
      Modelica.Mechanics.Rotational.Interfaces.Flange_b flange_b annotation(Placement(transformation(extent = {{94, -10}, {114, 10}})));
      Modelica.Mechanics.Rotational.Sensors.AngleSensor angleSensor annotation(Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {100, -46})));
    equation
      connect(Theta, throttleBody.Theta) annotation(Line(points = {{-120, 0}, {-92, 0}, {-92, 46}, {-80, 46}}, color = {0, 0, 127}, visible = true));
      connect(intakeManifold.P_m, throttleBody.Pm) annotation(Line(points = {{-57, -16}, {-52, -16}, {-52, -26}, {-88, -26}, {-88, 34}, {-80, 34}}, color = {0, 0, 127}, visible = true));
      connect(inductionToPowerDelay.m_a_delayed, torqueGeneration.m_a) annotation(Line(points = {{-6.894, -39.526}, {-1.894, -39.526}, {-1.894, 39.526}, {10.681, 39.526}}, color = {0, 0, 127}, visible = true, origin = {5.626, 6.474}));
      connect(cylinderAirCharge.m_a, inductionToPowerDelay.m_a) annotation(Line(points = {{-2.433, 14.226}, {0.067, 14.226}, {0.067, -14.226}, {2.299, -14.226}}, color = {0, 0, 127}, visible = true, origin = {-26.567, -18.826}));
      connect(cylinderAirCharge.clock, crankshaftPositionEvent.y) annotation(Line(points = {{-29.667, 31.6}, {-29.667, -15.8}, {59.333, -15.8}}, color = {128, 0, 255}, visible = true, origin = {-10.333, -48.2}));
      connect(inductionToPowerDelay.clock, crankshaftPositionEvent.y) annotation(Line(points = {{-20.423, 12.632}, {-20.423, -6.316}, {40.845, -6.316}}, color = {128, 0, 255}, visible = true, origin = {8.155, -57.684}));
      connect(torque.tau, torqueGeneration.T_torque_e) annotation(Line(points = {{38.4, 0}, {32, 0}, {32, 20}, {48, 20}, {48, 40}, {39.307, 40}}, color = {0, 0, 127}, visible = true));
      connect(torque.flange, inertia1.flange_a) annotation(Line(points = {{56, 0}, {64, 0}}));
      connect(inertia1.flange_b, speedSensor.flange) annotation(Line(points = {{80, 0}, {88.348, 0}, {88.348, -8}, {88, -8}}, visible = true));
      connect(inertia1.flange_b, flange_b) annotation(Line(points = {{80, 0}, {104, 0}}));
      connect(torqueGeneration.N, speedSensor.w) annotation(Line(points = {{16.307, 34}, {10, 34}, {10, -36}, {88, -36}, {88, -29}}, color = {0, 0, 127}, visible = true));
      connect(intakeManifold.N, speedSensor.w) annotation(Line(points = {{-80, -16}, {-86, -16}, {-86, -90}, {88, -90}, {88, -29}}, color = {0, 0, 127}, visible = true));
      connect(angleSensor.flange, inertia1.flange_b) annotation(Line(points = {{100, -36}, {100, 0}, {80, 0}}));
      connect(angleSensor.phi, crankshaftPositionEvent.angle) annotation(Line(points = {{100, -57}, {100, -64}, {72, -64}}, color = {0, 0, 127}));
      connect(throttleBody.m_ai_der, intakeManifold.m_ai_der) annotation(Line(visible = true, origin = {-69.739, 9.692}, points = {{12.739, 30.308}, {19.739, 30.308}, {19.739, 5.033}, {-15.848, 5.033}, {-15.848, -14.6}, {-10.261, -14.6}, {-10.261, -13.692}}, color = {0, 0, 127}));
      connect(intakeManifold.m_ao_der, cylinderAirCharge.m_ao_der) annotation(Line(visible = true, origin = {-54.5, -4.3}, points = {{-2.5, 0.3}, {0, 0.3}, {0, -0.3}, {2.5, -0.3}}, color = {0, 0, 127}));
      annotation(Documentation(info = "<html>

    <p>
Internal combustion engine
</p></html>"), Icon(graphics = {Rectangle(extent = {{-100, 100}, {100, -100}}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid), Polygon(points = {{-56, 62}, {36, 76}, {46, 52}, {-46, 38}, {-56, 62}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Polygon(points = {{58, 8}, {-34, -6}, {-34, -46}, {58, -32}, {58, 8}}, pattern = LinePattern.None, fillColor = {135, 135, 135}, fillPattern = FillPattern.Solid), Polygon(points = {{-58, -38}, {-34, -46}, {-34, -6}, {-16, 28}, {-36, 54}, {-46, 38}, {-56, 62}, {-76, 48}, {-58, 2}, {-58, -38}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid), Polygon(points = {{-16, 28}, {76, 42}, {58, 8}, {-34, -6}, {-16, 28}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Polygon(points = {{-36, 54}, {-16, 28}, {76, 42}, {56, 68}, {-36, 54}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Ellipse(extent = {{-18, 54}, {2, 34}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Ellipse(extent = {{40, 62}, {60, 42}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Ellipse(extent = {{12, 58}, {32, 38}}, lineColor = {0, 0, 255}, pattern = LinePattern.None, fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid), Line(points = {{-46, 38}, {-36, 54}, {40, 66}}), Text(extent = {{-100, -50}, {100, -90}}, lineColor = {0, 0, 255}, fillColor = {255, 255, 0}, fillPattern = FillPattern.Solid, textString = "%name")}), Diagram(coordinateSystem(extent = {{-100, -100}, {120, 70}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10})));
    end Engine;
    annotation(Documentation);
  end Components;
  annotation(preferredView = "info", uses(Modelica(version = "4.0.0")), __Wolfram(openClass = EngineThrottleControl.EngineThrottleControlExample), Documentation(info = "<html><p>In order to get the full experience of this example, you need a desktop Wolfram Language product. A free trial download is available at&nbsp;<a href=\"http://www.wolfram.com/mathematica/trial/\">www.wolfram.com/mathematica/trial/</a></p>

<p>For the full example, open the accompanying notebook <a href=\"modelica://EngineThrottleControl/EngineThrottleControl.nb\">EngineThrottleControl.nb</a>.</p></html>"), Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90.0, -90.0}, {90.0, 90.0}}, radius = 25), Text(visible = true, origin = {12.0, 20.8333}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-102.0, -70.8333}, {78.0, 29.1667}}, textString = "Ex", fontName = "Arial")}));
end EngineThrottleControl;
